#!/bin/csh 
### This script helps to build up the mapping with observed stations. The distributed parameter and data assimilation are 
### based on this mapping.
## Step 1.  edit the list.txt to input the stations. 
## Step 2. DOMAIN/Fulldom_finemesh.nc need to backup, since this part will modify it. 
## Step 3. make sure  aggFactor and dxRT are consistent with hydro.namelist.
## Step 4. run the script

set mpitasks = 1   ### total number of mpi tasks
set streamOrder = 1 ### set the threshold of stream order to increase the accuracy in searching the observation stations.
                    ## The small order stream will not be searched. 
set aggFactor = 20  ### the ratio of the resolution between land grid and land routing grid
set dxRT = 50      ### horizontal resolution in land routing domain (meters)


###--------------no need to change below -------------------

set NAMELIST = hydro.namelist
rm -f *.nc


if(! -e DOMAIN/Fulldom_finemesh.nc) then 
  echo "Error: no DOMAIN/Fulldom_finemesh.nc"
  exit
endif

if(! -e DOMAIN/geo_finemesh.nc) then 
  echo "Error: no DOMAIN/geo_finemesh.nc"
  exit
endif

### create the wrfinput_d01 
if(! -e DOMAIN/wrfinput_d01) then 
  if(-e DOMAIN/geo_finemesh.nc) ./cwrfInput.exe DOMAIN/geo_finemesh.nc DOMAIN/wrfinput_d01
endif

## this procedure will modify the Fulldom_finemesh.nc. Back up the original one. 
if (! -e DOMAIN/Fulldom_finemesh.nc.org) cp DOMAIN/Fulldom_finemesh.nc DOMAIN/Fulldom_finemesh.nc.org

## back up the related namelist files
if(! -e ${NAMELIST}.bak_d) cp $NAMELIST ${NAMELIST}.bak_d
if(! -e namelist.hrldas.bak_d) cp namelist.hrldas namelist.hrldas.bak_d

### step 1, when outStatic = 1 and run wrf_hydro.exe or dhydro.exe or NoahMP_dhydro.exe, it will generate the template files:
### seg_info.nc : the file for distributed parameters.
### factor.nc  : the file for the coefficients related to the surface runoff input.
### staticData.txt : the river node information.
sed "s,^.*FORC_TYP.*, FORC_TYP = 4,;" \
                namelist.hrldas >& namelist.hrldas.tmp; mv namelist.hrldas.tmp namelist.hrldas

sed "s,^.*outStatic .*, outStatic  = 1,;" \
                ${NAMELIST} >& ${NAMELIST}.tmp; mv ${NAMELIST}.tmp ${NAMELIST}
sed "s,^.*DABE .*, DABE  = 0,;" \
                ${NAMELIST} >& ${NAMELIST}.tmp; mv ${NAMELIST}.tmp ${NAMELIST}

mpirun -np $mpitasks ./wrf_hydro.exe
if($status != 0) then 
   echo "Error failed adt step 1 "
   exit
endif

### step 2: run the find.exe command to map the station with model. Based on the station lat-lon information to find its location inside the model.
### The output is station_list.txt. 
set nnTmp = `wc -l staticData.txt`
echo $nnTmp[1]
echo " ./find.exe $streamOrder $nnTmp[1] $aggFactor DOMAIN/geo_finemesh.nc "

## First to run this command find the watershed. 
## Since the obs could be in the middle of the watershed, this will find partial of the watershed.
./find.exe $streamOrder $nnTmp[1] $aggFactor DOMAIN/geo_finemesh.nc
if($status != 0) then 
   echo "Error failed adt step 2 "
   exit
endif

#### This section has been commented out. It is used for single watershed. 
#### The original list.txt does not include the end point. We add this point to list.txt and run find.exe again.
#### Second find the end point of the watershed and run the find.exe again. 
#### Now it will find all points of the watershed, extended to the domain boundary.
#if(-e bucket.CSV) then
#  set ll = `tail -10 bucket.CSV|head -1| sed "s/,/ /g"`
#  set lat = $ll[3]
#  set lon = $ll[2]
#  cp list.txt list.txt.bak_d
#  echo "$lat $lon 0 -9999 " >> list.txt
#  ./find.exe $streamOrder $nnTmp[1] $aggFactor DOMAIN/geo_finemesh.nc
#if($status != 0) then 
#   echo "Error failed adt step 3 "
#   exit
#endif
#endif

## Using the bucket_full2.CSV to cover the whole domain.
## Using the bucket2.CSV to cover the selected river.
## ./createBucket.exe  DOMAIN/Fulldom_finemesh.nc bucket_full2.CSV
## The "bucket2.CSV" is for single watershed. We commented it out. Here we will use the bucket2_full.CSV for the whole domain.
#   ./createBucket.exe  DOMAIN/Fulldom_finemesh.nc bucket2.CSV
#if($status != 0) then 
#   echo "Error failed adt step 4 "
#   exit
#endif

### step 3: We will update the Fulldom_finemesh.nc file. Each river node will have its own watershed (or called basain or bucket).
### This step only needs to do one time.
   ./createBucket.exe  DOMAIN/Fulldom_finemesh.nc bucket2_full.CSV 
if($status != 0) then 
   echo "Error failed adt step 5 "
   exit
endif


## Since the Fulldom_finemesh.nc has been changed. We need to get the new river node information, all CSV files. 
cp station_list.txt station_list.txt.bakd
mv list.txt.bak_d list.txt
rm -f *.nc 
mpirun -np $mpitasks ./wrf_hydro.exe
if($status != 0) then 
   echo "Error failed adt step 6 "
   exit
endif
set nnTmp = `wc -l staticData.txt`

echo "./find.exe $streamOrder $nnTmp[1] $aggFactor DOMAIN/geo_finemesh.nc" 

./find.exe $streamOrder $nnTmp[1] $aggFactor DOMAIN/geo_finemesh.nc
if($status != 0) then 
   echo "Error failed adt step 7 "
   exit
endif

### step 4: update the station information to seg_info.nc file. 
### Update the station information to seg_info.nc file. 
echo " ./updateSeg.exe DOMAIN/geo_finemesh.nc DOMAIN/Fulldom_finemesh.nc $dxRT  $streamOrder"
./updateSeg.exe DOMAIN/geo_finemesh.nc DOMAIN/Fulldom_finemesh.nc $dxRT $streamOrder 
if($status != 0) then 
   echo "Error failed adt step 8 "
   exit
endif

### step 5: Create the watershed in seg_info.nc for each observation. 
echo "./createSegMask.exe DOMAIN/Fulldom_finemesh.nc $aggFactor "
./createSegMask.exe DOMAIN/Fulldom_finemesh.nc $aggFactor 
if($status != 0) then 
   echo "Error failed adt step 9 "
   exit
endif

mv ${NAMELIST}.bak_d $NAMELIST 
mv namelist.hrldas.bak_d namelist.hrldas 

echo "Finished successfully ! "

### Now you are ready to adjust the parameters. 
## Step 5: It is an option to create the river geometry data. Current data is not very good. It needs to be improved in the future. 
##./river_geo.exe narivs.shp.nc DOMAIN/Fulldom_finemesh.nc DOMAIN/geo_finemesh.nc stream_geo.nc 100 seg_info.nc
